Public Class clsButtons
    Const pi = 3.141592654
    Const HSLMAX As Integer = 360 '***
    'H, S and L values can be 0 - HSLMAX
    '240 matches what is used by MS Win;
    'any number less than 1 byte is OK;
    'works best if it is evenly divisible by 6
    Private Class colorHSL
        'Portions of this class marked with *** are converted from
        'C/C++ routines for RGB/HSL conversion found in the
        'Microsoft Knowledge Base (PD sample code):
        'http://support.microsoft.com/support/kb/articles/Q29/2/40.asp
        'In addition to the language conversion, some internal
        'calculations have been modified and converted to FP math to
        'reduce rounding errors.
        'Conversion to VB and original code by
        'Dan Redding (bwsoft@revealed.net)
        'http://home.revealed.net/bwsoft
        'converted to VB.NET by Dave Andrews

        Const RGBMAX As Integer = 255 '***
        'R, G, and B value can be 0 - RGBMAX
        Const UNDEFINED As Integer = (HSLMAX * 2 / 3) '***
        'Hue is undefined if Saturation = 0 (greyscale)
        Public Hue As Integer
        Public Sat As Integer
        Public Lum As Integer
        Public Property ColorValue() As Color
            Get
                Return GetColor()
            End Get
            Set(ByVal Value As Color)
                Hue = (Value.GetHue / 360) * HSLMAX
                Sat = Value.GetSaturation * HSLMAX
                Lum = Value.GetBrightness * HSLMAX
            End Set
        End Property

        Public Sub New(ByVal C As Color)
            ColorValue = C
        End Sub
        Public Sub New(ByVal R As Integer, ByVal G As Integer, ByVal B As Integer)
            ColorValue = Color.FromArgb(R, G, B)
        End Sub
        Public Sub New()
            ColorValue = Color.Black
        End Sub
        Public Sub FromHSL(ByVal H As Integer, ByVal S As Integer, ByVal L As Integer)
            Dim C As Color
            Hue = H
            Sat = S
            Lum = L
            C = GetColor()
            ColorValue = C
        End Sub
        Private Function HuetoRGB(ByVal M1 As Single, ByVal M2 As Single, ByVal Hue As Single) As Single
            '***
            'Utility function for HSLtoLong
            'Range check
            If Hue < 0 Then
                Hue = Hue + HSLMAX
            ElseIf Hue > HSLMAX Then
                Hue = Hue - HSLMAX
            End If
            'Return r, g, or b value from parameters
            Select Case Hue 'Values get progressively larger.
                'Only the first true condition will execute
            Case Is < (HSLMAX / 6)
                    HuetoRGB = (M1 + (((M2 - M1) * Hue + (HSLMAX / 12)) / (HSLMAX / 6)))
                Case Is < (HSLMAX / 2)
                    HuetoRGB = M2
                Case Is < (HSLMAX * 2 / 3)
                    HuetoRGB = (M1 + (((M2 - M1) * ((HSLMAX * 2 / 3) - Hue) + (HSLMAX / 12)) / (HSLMAX / 6)))
                Case Else
                    HuetoRGB = M1
            End Select
        End Function
        Private Function GetColor() As Color
            Dim R As Single, G As Single, B As Single
            Dim M1 As Single, M2 As Single
            If CInt(Sat) = 0 Then 'Greyscale
                R = (Lum * RGBMAX) / HSLMAX 'luminescence,
                'converted to the proper range
                G = R 'All RGB values same in greyscale
                B = R
                If CInt(Hue) <> UNDEFINED Then
                    'This is technically an error.
                    'The RGBtoHSL routine will always return
                    'Hue = UNDEFINED (160 when HSLMAX is 240)
                    'when Sat = 0.
                    'if you are writing a color mixer and
                    'letting the user input color values,
                    'you may want to set Hue = UNDEFINED
                    'in this case.
                End If
            Else
                'Get the "Magic Numbers"
                If Lum <= HSLMAX / 2 Then
                    M2 = (Lum * (HSLMAX + Sat) + 0.5) / HSLMAX
                Else
                    M2 = Lum + Sat - ((Lum * Sat) + 0.5) / HSLMAX
                End If

                M1 = 2 * Lum - M2

                'get R, G, B; change units from HSLMAX range
                'to RGBMAX range
                R = (HuetoRGB(M1, M2, Hue + (HSLMAX / 3)) * RGBMAX + 0.5) / HSLMAX
                G = (HuetoRGB(M1, M2, Hue) * RGBMAX + 0.5) / HSLMAX
                B = (HuetoRGB(M1, M2, Hue - (HSLMAX / 3)) * RGBMAX + 0.5) / HSLMAX

            End If

            GetColor = Color.FromArgb(CInt(R), CInt(G), CInt(B))

        End Function
    End Class
    Public Enum BTNState As Integer
        ButtonUp = 0
        ButtonDown = 1
    End Enum
    Dim Transparent As Boolean = False
    Dim Radius As Integer = 0
    Dim Bevel As Integer = 1
    Dim Pic As PictureBox
    Dim TextX As Integer = 0
    Dim TextY As Integer = 0
    Dim EmbossText As Boolean = True
    Dim Toggle As Boolean = False
    Dim Back As Bitmap
    Dim Front As Bitmap
    Dim BMP As Bitmap
    Dim CurState As BTNState = BTNState.ButtonUp
    Dim PrevState As BTNState = BTNState.ButtonUp
    Dim BTNText As String
    Dim PS As Integer
    Dim BTNFont As Font
    Public Property Enabled() As Boolean
        Get
            Return Pic.Enabled
        End Get
        Set(ByVal Value As Boolean)
            Pic.Enabled = Value
        End Set
    End Property
    Public Property ButtonState() As BTNState
        Get
            Return CurState
        End Get
        Set(ByVal Value As BTNState)
            PrevState = CurState
            CurState = Value
            Select Case CurState
                Case BTNState.ButtonUp : DrawUp()
                Case BTNState.ButtonDown : DrawDown()
            End Select
        End Set
    End Property
    Public Property Text() As String
        Get
            Return BTNText
        End Get
        Set(ByVal Value As String)
            BTNText = Value
            ButtonState = CurState
        End Set
    End Property
    Public Property Font() As Font
        Get
            Return BtnFont
        End Get
        Set(ByVal Value As Font)
            BtnFont = Value
            ButtonState = CurState
        End Set
    End Property
    Public Sub New(ByVal bPic As PictureBox, ByVal bText As String, Optional ByVal bTransparent As Boolean = False, Optional ByVal bRadius As Integer = 0, Optional ByVal bBevel As Integer = 1, Optional ByVal bPercentShadow As Integer = 50, Optional ByVal bEmbossText As Boolean = True, Optional ByVal bTextOffsetX As Integer = 0, Optional ByVal bTextOffsetY As Integer = 0, Optional ByVal UseEnterLeave As Boolean = True, Optional ByVal bToggle As Boolean = False, Optional ByVal bEnabled As Boolean = True, Optional ByVal bFont As Font = Nothing)
        Pic = bPic
        BTNText = bText
        Transparent = bTransparent
        Radius = bRadius
        Bevel = bBevel
        Pic.Enabled = bEnabled
        EmbossText = bEmbossText
        TextX = bTextOffsetX
        TextY = bTextOffsetY
        Toggle = bToggle
        PS = bPercentShadow
        If bFont Is Nothing Then
            BTNFont = Pic.Font
        Else
            BTNFont = bFont
        End If
        If Bevel > (Pic.Width / 2) Then Bevel = (Pic.Width / 2) - 1
        If Bevel > (Pic.Height / 2) Then Bevel = (Pic.Height / 2) - 1
        If Radius > (Pic.Width / 2) Then Radius = (Pic.Width / 2) - 1
        If Radius > (Pic.Height / 2) Then Radius = (Pic.Height / 2) - 1
        AddHandler Pic.MouseDown, AddressOf MouseDown
        AddHandler Pic.MouseUp, AddressOf MouseUp
        AddHandler Pic.Paint, AddressOf Paint
        AddHandler Pic.LocationChanged, AddressOf LocationChanged
        AddHandler Pic.SizeChanged, AddressOf SizeChanged
        If UseEnterLeave Then
            AddHandler Pic.MouseEnter, AddressOf MouseEnter
            AddHandler Pic.MouseLeave, AddressOf MouseLeave
        End If
        Redraw()
    End Sub
    Public Sub Redraw()
        SetBackImage()
        SetFrontImage()
        ButtonState = CurState
    End Sub
    Public Sub dispose()
        RemoveHandler Pic.MouseDown, AddressOf MouseDown
        RemoveHandler Pic.MouseUp, AddressOf MouseUp
        RemoveHandler Pic.Paint, AddressOf Paint
        RemoveHandler Pic.MouseEnter, AddressOf MouseEnter
        RemoveHandler Pic.MouseLeave, AddressOf MouseLeave
    End Sub

    Private Function Brighten(ByVal C As Color, ByVal Percent As Single) As Color
        Dim HSL As New colorHSL(C)
        Dim L As Single
        If Percent <= 0 Then
            Brighten = C
            Exit Function
        End If
        L = HSL.Lum + (HSLMAX * Percent)
        If L > HSLMAX Then L = HSLMAX
        HSL.Lum = L
        Brighten = HSL.ColorValue
    End Function
    Private Function Darken(ByVal C As Color, ByVal Percent As Single) As Color

        Dim HSL As New colorHSL(C)
        Dim L As Single
        If Percent <= 0 Then
            Darken = C
            Exit Function
        End If

        L = HSL.Lum - (HSLMAX * Percent)
        If L < 0 Then L = 0
        HSL.Lum = L
        Darken = HSL.ColorValue
    End Function
    Private Function Blend(ByVal C1 As Color, ByVal C2 As Color, ByVal P1 As Single) As Color
        Dim R1 As Integer = C1.R * P1
        Dim G1 As Integer = C1.G * P1
        Dim B1 As Integer = C1.B * P1
        Dim R2 As Integer = C2.R * (1 - P1)
        Dim G2 As Integer = C2.G * (1 - P1)
        Dim B2 As Integer = C2.B * (1 - P1)
        Blend = Color.FromArgb((R1 + R2), (G1 + G2), (B1 + B2))

    End Function
    Private Function Invert(ByVal C As Color) As Color
        Invert = Color.FromArgb(255 - C.R, 255 - C.G, 255 - C.B)
    End Function
    Private Sub SetBackImage()
        Dim tCont As Control = Pic.Parent
        Dim tBMP As Bitmap
        Dim IMG As Image
        Dim i As Integer
        Dim j As Integer
        Try
            tBMP = tCont.BackgroundImage
        Catch
            tBMP = Pic.FindForm.BackgroundImage
        End Try
        Back = New Bitmap(Pic.Width, Pic.Height)
        Try
            For i = 0 To Back.Width - 1
                For j = 0 To Back.Height - 1
                    Back.SetPixel(i, j, tBMP.GetPixel(i + Pic.Left, j + Pic.Top))
                Next
            Next
        Catch
            For i = 0 To Back.Width - 1
                For j = 0 To Back.Height - 1
                    Back.SetPixel(i, j, tCont.BackColor)
                Next
            Next
        End Try

    End Sub
    Private Sub SetFrontImage()
        If Transparent Then
            Front = New Bitmap(Back)
        Else
            Try
                Front = New Bitmap(Pic.Image)
            Catch
                Dim i As Integer
                Dim j As Integer
                Front = New Bitmap(Pic.Width, Pic.Height)
                For i = 0 To Front.Width - 1
                    For j = 0 To Front.Height - 1
                        Front.SetPixel(i, j, Pic.BackColor)
                    Next
                Next
            End Try
        End If
    End Sub
    Private Sub DrawBackCorners()
        Dim i As Integer
        Dim j As Integer
        For i = 0 To Radius - 1
            For j = 0 To Radius - 1
                BMP.SetPixel(i, j, Back.GetPixel(i, j))
                BMP.SetPixel(BMP.Width - Radius + i, j, Back.GetPixel(BMP.Width - Radius + i, j))
                BMP.SetPixel(i, BMP.Height - Radius + j, Back.GetPixel(i, BMP.Height - Radius + j))
                BMP.SetPixel(BMP.Width - Radius + i, BMP.Height - Radius + j, Back.GetPixel(BMP.Width - Radius + i, BMP.Height - Radius + j))
            Next
        Next
    End Sub
    Private Sub DrawUp()
        Dim i As Integer
        Dim j As Integer
        Dim P As Single
        Dim C As Color
        'First we start with a clean playing field
        BMP = New Bitmap(Front)
        For j = 0 To Bevel - 1
            P = ((Bevel - (j + 1)) * (PS / Bevel)) / 100
            ''darken the bottom edge and lighten the top edge
            For i = 0 + j To (BMP.Width - 1) - j
                C = Front.GetPixel(i, j)
                BMP.SetPixel(i, j, Brighten(C, P))
                C = Front.GetPixel(i, (BMP.Height - 1) - j)
                BMP.SetPixel(i, (BMP.Height - 1) - j, Darken(C, P))
            Next i
            ''darken the right edge and lighten the left edge
            For i = 0 + j To (BMP.Height - 1) - j
                C = Front.GetPixel(j, i)
                BMP.SetPixel(j, i, Brighten(C, P))
                C = Front.GetPixel((BMP.Width - 1) - j, i)
                BMP.SetPixel((BMP.Width - 1) - j, i, Darken(C, P))
            Next i
        Next j
        If Radius > 0 Then
            Dim nX As Integer
            Dim nY As Integer
            Dim r As Integer
            Dim d As Single
            j = 0
            DrawBackCorners()
            For r = Radius - 1 To 0 Step -1
                P = ((Bevel - (j + 1)) * (PS / Bevel)) / 100
                For d = 0 To pi / 2 Step pi / (Radius * 7)
                    'Top Left
                    nX = Radius - (Math.Cos(d) * r) - 1
                    nY = Radius - (Math.Sin(d) * r) - 1
                    If j <= Bevel - 1 Then
                        C = Brighten(Front.GetPixel(nX, nY), P)
                    Else
                        C = Front.GetPixel(nX, nY)
                    End If
                    BMP.SetPixel(nX, nY, C)
                    'Bottom Right
                    nX = BMP.Width - Radius + (Math.Cos(d) * r)
                    nY = BMP.Height - Radius + (Math.Sin(d) * r)
                    If j <= Bevel - 1 Then
                        C = Darken(Front.GetPixel(nX, nY), P)
                    Else
                        C = Front.GetPixel(nX, nY)
                    End If
                    BMP.SetPixel(nX, nY, C)
                    'Top Right
                    nX = BMP.Width - Radius + (Math.Cos(d) * r)
                    nY = Radius - (Math.Sin(d) * r) - 1
                    If j <= Bevel - 1 Then
                        C = Brighten(Front.GetPixel(nX, nY), P)
                        C = Blend(C, Darken(Front.GetPixel(nX, nY), P), d / (pi / 2))
                    Else
                        C = Front.GetPixel(nX, nY)
                    End If
                    BMP.SetPixel(nX, nY, C)
                    'Bottom Left
                    nX = Radius - (Math.Cos(d) * r) - 1
                    nY = BMP.Height - Radius + (Math.Sin(d) * r)
                    If j <= Bevel - 1 Then
                        C = Darken(Front.GetPixel(nX, nY), P)
                        C = Blend(C, Brighten(Front.GetPixel(nX, nY), P), d / (pi / 2))
                    Else
                        C = Front.GetPixel(nX, nY)
                    End If
                    BMP.SetPixel(nX, nY, C)
                Next
                j += 1
            Next
        End If
        Pic.CreateGraphics.DrawImage(BMP, 0, 0)
        DrawText(Pic.CreateGraphics)
    End Sub
    Private Sub DrawDown()
        Dim i As Integer
        Dim j As Integer
        Dim P As Single
        Dim C As Color
        'First we start with a clean playing field
        BMP = New Bitmap(Front)
        ''darken the top edge and lighten the bottom edge
        For j = 0 To Bevel - 1
            P = ((Bevel - (j + 1)) * (PS / Bevel)) / 100
            For i = 0 + j To (BMP.Width - 1) - j
                C = Front.GetPixel(i, j)
                BMP.SetPixel(i, j, Darken(C, P))
                C = Front.GetPixel(i, (BMP.Height - 1) - j)
                BMP.SetPixel(i, (BMP.Height - 1) - j, Brighten(C, P))
            Next i
            ''darken the left edge and lighten the right edge
            For i = 0 + j To (BMP.Height - 1) - j
                C = Front.GetPixel(j, i)
                BMP.SetPixel(j, i, Darken(C, P))
                C = Front.GetPixel((BMP.Width - 1) - j, i)
                BMP.SetPixel((BMP.Width - 1) - j, i, Brighten(C, P))
            Next i
        Next j
        If Radius > 0 Then
            Dim nX As Integer
            Dim nY As Integer
            Dim r As Integer
            Dim d As Single
            j = 0
            DrawBackCorners()
            For r = Radius - 1 To 0 Step -1
                P = ((Bevel - (j + 1)) * (PS / Bevel)) / 100
                For d = 0 To pi / 2 Step pi / (Radius * 7)
                    'Top Left
                    nX = Radius - (Math.Cos(d) * r) - 1
                    nY = Radius - (Math.Sin(d) * r) - 1
                    If j <= Bevel - 1 Then
                        C = Darken(Front.GetPixel(nX, nY), P)
                    Else
                        C = Front.GetPixel(nX, nY)
                    End If
                    BMP.SetPixel(nX, nY, C)
                    'Bottom Right
                    nX = BMP.Width - Radius + (Math.Cos(d) * r)
                    nY = BMP.Height - Radius + (Math.Sin(d) * r)
                    If j <= Bevel - 1 Then
                        C = Brighten(Front.GetPixel(nX, nY), P)
                    Else
                        C = Front.GetPixel(nX, nY)
                    End If
                    BMP.SetPixel(nX, nY, C)
                    'Top Right
                    nX = BMP.Width - Radius + (Math.Cos(d) * r)
                    nY = Radius - (Math.Sin(d) * r) - 1
                    If j <= Bevel - 1 Then
                        C = Darken(Front.GetPixel(nX, nY), P)
                        C = Blend(C, Brighten(Front.GetPixel(nX, nY), P), d / (pi / 2))
                    Else
                        C = Front.GetPixel(nX, nY)
                    End If
                    BMP.SetPixel(nX, nY, C)
                    'Bottom Left
                    nX = Radius - (Math.Cos(d) * r) - 1
                    nY = BMP.Height - Radius + (Math.Sin(d) * r)
                    If j <= Bevel - 1 Then
                        C = Brighten(Front.GetPixel(nX, nY), P)
                        C = Blend(C, Darken(Front.GetPixel(nX, nY), P), d / (pi / 2))
                    Else
                        C = Front.GetPixel(nX, nY)
                    End If
                    BMP.SetPixel(nX, nY, C)
                Next
                j += 1
            Next
        End If
        Pic.CreateGraphics.DrawImage(BMP, 0, 0)
        DrawText(Pic.CreateGraphics)
    End Sub
    Private Sub DrawEnter()
        Dim i As Integer
        Dim j As Integer
        Dim P As Single
        Dim C As Color
        'First we start with a clean playing field
        BMP = New Bitmap(Front)
        'Brighten Outer edges and darken inner
        For j = 0 To Bevel - 1
            P = ((Bevel - (j + 1)) * (PS / Bevel)) / 100
            If j < ((Bevel - 1) / 2) Then
                For i = 0 + j To (BMP.Width - 1) - j
                    C = Front.GetPixel(i, j)
                    BMP.SetPixel(i, j, Brighten(C, P))
                    C = Front.GetPixel(i, (BMP.Height - 1) - j)
                    BMP.SetPixel(i, (BMP.Height - 1) - j, Brighten(C, P))
                Next i
                For i = 0 + j To (BMP.Height - 1) - j
                    C = Front.GetPixel(j, i)
                    BMP.SetPixel(j, i, Brighten(C, P))
                    C = Front.GetPixel((BMP.Width - 1) - j, i)
                    BMP.SetPixel((BMP.Width - 1) - j, i, Brighten(C, P))
                Next i
            Else
                For i = 0 + j To (BMP.Width - 1) - j
                    C = Front.GetPixel(i, j)
                    BMP.SetPixel(i, j, Darken(C, P))
                    C = Front.GetPixel(i, (BMP.Height - 1) - j)
                    BMP.SetPixel(i, (BMP.Height - 1) - j, Darken(C, P))
                Next i
                For i = 0 + j To (BMP.Height - 1) - j
                    C = Front.GetPixel(j, i)
                    BMP.SetPixel(j, i, Darken(C, P))
                    C = Front.GetPixel((BMP.Width - 1) - j, i)
                    BMP.SetPixel((BMP.Width - 1) - j, i, Darken(C, P))
                Next i
            End If
        Next j
        If Radius > 0 Then
            Dim nX As Integer
            Dim nY As Integer
            Dim r As Integer
            Dim d As Single
            j = 0
            DrawBackCorners()
            For r = Radius - 1 To 0 Step -1
                P = ((Bevel - (j + 1)) * (PS / Bevel)) / 100
                For d = 0 To pi / 2 Step pi / (Radius * 7)
                    If j < ((Bevel - 1) / 2) Then
                        'Top Left
                        nX = Radius - (Math.Cos(d) * r) - 1
                        nY = Radius - (Math.Sin(d) * r) - 1
                        C = Brighten(Front.GetPixel(nX, nY), P)
                        BMP.SetPixel(nX, nY, C)
                        'Bottom Right
                        nX = BMP.Width - Radius + (Math.Cos(d) * r)
                        nY = BMP.Height - Radius + (Math.Sin(d) * r)
                        C = Brighten(Front.GetPixel(nX, nY), P)
                        BMP.SetPixel(nX, nY, C)
                        'Top Right
                        nX = BMP.Width - Radius + (Math.Cos(d) * r)
                        nY = Radius - (Math.Sin(d) * r) - 1
                        P = ((Bevel - (j + 1)) * (PS / Bevel)) / 100
                        C = Brighten(Front.GetPixel(nX, nY), P)
                        BMP.SetPixel(nX, nY, C)
                        'Bottom Left
                        nX = Radius - (Math.Cos(d) * r) - 1
                        nY = BMP.Height - Radius + (Math.Sin(d) * r)
                        C = Brighten(Front.GetPixel(nX, nY), P)
                        BMP.SetPixel(nX, nY, C)
                    ElseIf j <= Bevel - 1 Then
                        'Top Left
                        nX = Radius - (Math.Cos(d) * r) - 1
                        nY = Radius - (Math.Sin(d) * r) - 1
                        C = Darken(Front.GetPixel(nX, nY), P)
                        BMP.SetPixel(nX, nY, C)
                        'Bottom Right
                        nX = BMP.Width - Radius + (Math.Cos(d) * r)
                        nY = BMP.Height - Radius + (Math.Sin(d) * r)
                        C = Darken(Front.GetPixel(nX, nY), P)
                        BMP.SetPixel(nX, nY, C)
                        'Top Right
                        nX = BMP.Width - Radius + (Math.Cos(d) * r)
                        nY = Radius - (Math.Sin(d) * r) - 1
                        C = Darken(Front.GetPixel(nX, nY), P)
                        BMP.SetPixel(nX, nY, C)
                        'Bottom Left
                        nX = Radius - (Math.Cos(d) * r) - 1
                        nY = BMP.Height - Radius + (Math.Sin(d) * r)
                        C = Darken(Front.GetPixel(nX, nY), P)
                        BMP.SetPixel(nX, nY, C)
                    Else
                        'Top Left
                        nX = Radius - (Math.Cos(d) * r) - 1
                        nY = Radius - (Math.Sin(d) * r) - 1
                        C = Front.GetPixel(nX, nY)
                        BMP.SetPixel(nX, nY, C)
                        'Bottom Right
                        nX = BMP.Width - Radius + (Math.Cos(d) * r)
                        nY = BMP.Height - Radius + (Math.Sin(d) * r)
                        C = Front.GetPixel(nX, nY)
                        BMP.SetPixel(nX, nY, C)
                        'Top Right
                        nX = BMP.Width - Radius + (Math.Cos(d) * r)
                        nY = Radius - (Math.Sin(d) * r) - 1
                        C = Front.GetPixel(nX, nY)
                        BMP.SetPixel(nX, nY, C)
                        'Bottom Left
                        nX = Radius - (Math.Cos(d) * r) - 1
                        nY = BMP.Height - Radius + (Math.Sin(d) * r)
                        C = Front.GetPixel(nX, nY)
                        BMP.SetPixel(nX, nY, C)
                    End If
                Next
                j += 1
            Next
        End If
        Pic.CreateGraphics.DrawImage(BMP, 0, 0)
        DrawText(Pic.CreateGraphics)
    End Sub
    Private Sub DrawText(ByVal G As Graphics)
        Dim Size As SizeF
        Dim x As Single
        Dim y As Single
        Dim B As SolidBrush
        'we have to find out EXACTLY how to align text center on the button
        Size = G.MeasureString(Text, BTNFont)
        x = ((Pic.Width - Size.Width) / 2) + TextX
        y = ((Pic.Height - Size.Height) / 2) + TextY
        If CurState = BTNState.ButtonDown Then
            x += 1
            y += 1
        End If
        If EmbossText Then
            B = New SolidBrush(Invert(Pic.ForeColor))
            G.DrawString(Text, BTNFont, B, x + 1, y + 1)
        End If
        B = New SolidBrush(Pic.ForeColor)
        G.DrawString(Text, BTNFont, B, x, y)
    End Sub
    Private Sub MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        If Toggle Then
            If ButtonState = BTNState.ButtonDown Then
                ButtonState = BTNState.ButtonUp
            Else
                ButtonState = BTNState.ButtonDown
            End If
        Else
            ButtonState = BTNState.ButtonUp
        End If
    End Sub
    Private Sub MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        If Not Toggle Then ButtonState = BTNState.ButtonDown
    End Sub
    Private Sub MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs)
        DrawEnter()
    End Sub
    Private Sub MouseLeave(ByVal sender As Object, ByVal e As System.EventArgs)
        ButtonState = CurState
    End Sub

    Private Sub Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs)
        ButtonState = CurState
        e.Graphics.DrawImage(BMP, 0, 0)
        DrawText(e.Graphics)
    End Sub
    Private Sub SizeChanged(ByVal sender As Object, ByVal e As System.EventArgs)
        Redraw()
    End Sub

    Private Sub LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs)
        Redraw()
    End Sub
End Class
